
!
! BB2NCDF
!
! The program BB2NCDF reads the parameters contained
! in the binary files basin_b/basin_i and writes many DTM files in a netCDF
! output file.
!
!-----------------------------------------------------------------------------
subroutine bb2ncdf_sr

use netcdf
use mpar
use mbbio

implicit none

! This is the name of the data file we will create.
character (len = *), parameter :: FILE_NAME = "CATHY_prepro.nc"
character (len = *), parameter :: FILE_NAME2 = "CATHY_prepro_cut.nc"
character (len = *), parameter :: title_dataset = "HAP pre-processor variables for the CATHY model."

integer(kind=ISP) :: file_id, status

integer(kind=ISP) i,j,i_basin

! We are writing 2D data (6 variables), in a UTM grid. We will need two netCDF dimensions.
integer(kind=ISP), parameter :: ndims = 2
integer(kind=ISP), parameter :: nvars = 6

integer(kind=ISP) , dimension(nvars) :: var_id
integer(kind=ISP) , dimension(ndims) :: dimids

character (len = *), parameter :: LONG_NAME = "long_name"
character (len = *), parameter :: DEM_NAME = "digital elevation model"
character (len = *), parameter :: AINF_NAME = "inflow drainage area"
character (len = *), parameter :: LS_NAME = "local slope"       
character (len = *), parameter :: HCID_NAME = "hillslope/channel ID"
character (len = *), parameter :: UTM_Y_NAME = "UTM coordinates along y-direction" 
character (len = *), parameter :: UTM_X_NAME = "UTM coordinates along x-direction "

character (len = *), parameter :: UNITS = "units"
character (len = *), parameter :: DEM_UNITS = "m"
character (len = *), parameter :: AINF_UNITS = "m^2"
character (len = *), parameter :: LS_UNITS = "m/m"
character (len = *), parameter :: HCID_UNITS = "0 for hillslope ; 1 for channel"
character (len = *), parameter :: UTM_Y_UNITS = "meters_north"
character (len = *), parameter :: UTM_X_UNITS = "meters_east"

real(kind=REP), parameter :: DEM_FillValue = 1e+36
real(kind=REP), parameter :: AINF_FillValue =  1e+36
real(kind=REP), parameter :: LS_FillValue =  1e+36
real(kind=REP), parameter :: HCID_FillValue =  1e+36
real(kind=REP), parameter :: UTM_Y_FillValue =  1e+36
real(kind=REP), parameter :: UTM_X_FillValue =  1e+36

! Pre_processor main output fields
real(kind=REP),allocatable :: utm_x(:,:)
real(kind=REP),allocatable :: utm_y(:,:)
real(kind=REP),allocatable :: topo(:,:)
real(kind=REP),allocatable :: ainf(:,:)
real(kind=REP),allocatable :: ls1(:,:)
real(kind=REP),allocatable :: ls2(:,:)
real(kind=REP),allocatable :: ls(:,:)
real(kind=REP),allocatable :: w1(:,:)
real(kind=REP),allocatable :: w2(:,:)
real(kind=ISP),allocatable :: hcid(:,:)
integer(kind=ISP),allocatable :: mask(:,:)
! Allocate memory

allocate(utm_x(N,M))
allocate(utm_y(N,M))

allocate(topo(N,M))
allocate(ainf(N,M))
allocate(ls1(N,M))
allocate(ls2(N,M))
allocate(ls(N,M))
allocate(w1(N,M))
allocate(w2(N,M))
allocate(hcid(N,M))
allocate(mask(N,M))

call rparfile('hap.in')
call load_dtm("basin_b","basin_i")
!CMS
if (cdc.eq.0) then
   do i=1,N
      do j=1,M
         i_basin=(i-1)*M+j
         if (dtm_index_pr(i_basin) == 0)  then
            utm_x(i,j) = 1e+36
            utm_y(i,j) = 1e+36 
            topo(i,j) = 1e+36
            ainf(i,j) = 1e+36
            ls1(i,j)  = 1e+36
            ls2(i,j)  = 1e+36
            ls(i,j)   = 1e+36
            w1(i,j)   = 1e+36
            w2(i,j)   = 1e+36
            hcid(i,j) = 1e+36
         else
            utm_x(i,j) =xllcorner + (i-1)*delta_x
            utm_y(i,j) =yllcorner + (j-1)*delta_y 
            topo(i,j) = dtm_quota(i_basin)
            ainf(i,j) = dtm_A_inflow(i_basin)
            hcid(i,j) = dtm_hcID(i_basin)
            ls1(i,j)  = dtm_local_slope_1(i_basin)
            ls2(i,j)  = dtm_local_slope_2(i_basin) 
            w1(i,j)   = dtm_w_1(i_basin)
            w2(i,j)   = dtm_w_2(i_basin)
            ls(i,j)   = w1(i,j) * ls1(i,j) + w2(i,j) * ls2(i,j)
         end if
      end do
   end do
   status = nf90_create(FILE_NAME, NF90_CLOBBER, file_id)

   status = nf90_put_att(file_id,NF90_GLOBAL,'title',trim(title_dataset))

   status = nf90_def_dim(file_id, "NROW", N, dimids(1))
   status = nf90_def_dim(file_id, "NCOL", M, dimids(2))

   status = nf90_def_var(file_id, "UTM_X", NF90_DOUBLE, dimids, var_id(1))
   status = nf90_def_var(file_id, "UTM_Y", NF90_DOUBLE, dimids, var_id(2))
   status = nf90_def_var(file_id, "DEM", NF90_DOUBLE, dimids, var_id(3))
   status = nf90_def_var(file_id, "AINF", NF90_DOUBLE, dimids, var_id(4))
   status = nf90_def_var(file_id, "LS", NF90_DOUBLE, dimids, var_id(5))
   status = nf90_def_var(file_id, "HCID", NF90_DOUBLE, dimids, var_id(6))

   status = nf90_put_att(file_id, var_id(1), LONG_NAME, UTM_X_NAME)
   status = nf90_put_att(file_id, var_id(2), LONG_NAME, UTM_Y_NAME)
   status = nf90_put_att(file_id, var_id(3), LONG_NAME, DEM_NAME)
   status = nf90_put_att(file_id, var_id(4), LONG_NAME, AINF_NAME)
   status = nf90_put_att(file_id, var_id(5), LONG_NAME, LS_NAME)
   status = nf90_put_att(file_id, var_id(6), LONG_NAME, HCID_NAME)

   status = nf90_put_att(file_id, var_id(1), UNITS, UTM_X_UNITS)
   status = nf90_put_att(file_id, var_id(2), UNITS, UTM_Y_UNITS)
   status = nf90_put_att(file_id, var_id(3), UNITS, DEM_UNITS)
   status = nf90_put_att(file_id, var_id(4), UNITS, AINF_UNITS)
   status = nf90_put_att(file_id, var_id(5), UNITS, LS_UNITS)
   status = nf90_put_att(file_id, var_id(6), UNITS, HCID_UNITS)

   status = nf90_put_att(file_id, var_id(1),'_FillValue', UTM_X_FillValue)
   status = nf90_put_att(file_id, var_id(2),'_FillValue', UTM_Y_FillValue)
   status = nf90_put_att(file_id, var_id(3),'_FillValue', DEM_FillValue)
   status = nf90_put_att(file_id, var_id(4),'_FillValue', AINF_FillValue)
   status = nf90_put_att(file_id, var_id(5),'_FillValue', LS_FillValue)
   status = nf90_put_att(file_id, var_id(6),'_FillValue', HCID_FillValue)

   status = nf90_enddef(file_id) 
   status = nf90_put_var(file_id, var_id(1), utm_x,  &
        start = (/1, 1/), count = (/N, M/))
   status = nf90_put_var(file_id, var_id(2), utm_y,  &
        start = (/1, 1/), count = (/N, M/))
   status = nf90_put_var(file_id, var_id(3), topo)
   status = nf90_put_var(file_id, var_id(4), ainf)
   status = nf90_put_var(file_id, var_id(5), ls)
   status = nf90_put_var(file_id, var_id(6), hcid)
   status = nf90_close(file_id)
else
   do i=1,N
      do j=1,M
         i_basin=(i-1)*M+j
         if (dtm_index_pr(i_basin) == 0)  then
            utm_x(i,j) = 1e+36
            utm_y(i,j) = 1e+36 
            topo(i,j) = 1e+36
            ainf(i,j) = 1e+36
            ls1(i,j)  = 1e+36
            ls2(i,j)  = 1e+36
            ls(i,j)   = 1e+36
            w1(i,j)   = 1e+36
            w2(i,j)   = 1e+36
            hcid(i,j) = 1e+36
            mask(i,j) = -9999
         else
            utm_x(i,j) =xllcorner + (i-1)*delta_x
            utm_y(i,j) =yllcorner + (j-1)*delta_y
            mask(i,j) = dtm_mask(i_basin) 
         if (mask(i,j) .eq. 1) then
            topo(i,j) = dtm_quota(i_basin)*mask(i,j)
            ainf(i,j) = dtm_A_inflow(i_basin)*mask(i,j)
            hcid(i,j) = dtm_hcID(i_basin)*mask(i,j)
            ls1(i,j)  = dtm_local_slope_1(i_basin)
            ls2(i,j)  = dtm_local_slope_2(i_basin) 
            w1(i,j)   = dtm_w_1(i_basin)
            w2(i,j)   = dtm_w_2(i_basin)
            ls(i,j)   = (w1(i,j) * ls1(i,j) + w2(i,j) * ls2(i,j))*mask(i,j)
         else
            topo(i,j) = 1e+36
            ainf(i,j) = 1e+36
            hcid(i,j) = 1e+36
            ls1(i,j)  = 1e+36
            ls2(i,j)  = 1e+36
            w1(i,j)   = 1e+36
            w2(i,j)   = 1e+36
            ls(i,j)   =  1e+36
         end if
      end if
   end do
end do
status = nf90_create(FILE_NAME2, NF90_CLOBBER, file_id)

status = nf90_put_att(file_id,NF90_GLOBAL,'title',trim(title_dataset))

status = nf90_def_dim(file_id, "NROW", N, dimids(1))
status = nf90_def_dim(file_id, "NCOL", M, dimids(2))

status = nf90_def_var(file_id, "UTM_X", NF90_DOUBLE, dimids, var_id(1))
status = nf90_def_var(file_id, "UTM_Y", NF90_DOUBLE, dimids, var_id(2))
status = nf90_def_var(file_id, "DEM", NF90_DOUBLE, dimids, var_id(3))
status = nf90_def_var(file_id, "AINF", NF90_DOUBLE, dimids, var_id(4))
status = nf90_def_var(file_id, "LS", NF90_DOUBLE, dimids, var_id(5))
status = nf90_def_var(file_id, "HCID", NF90_DOUBLE, dimids, var_id(6))

status = nf90_put_att(file_id, var_id(1), LONG_NAME, UTM_X_NAME)
status = nf90_put_att(file_id, var_id(2), LONG_NAME, UTM_Y_NAME)
status = nf90_put_att(file_id, var_id(3), LONG_NAME, DEM_NAME)
status = nf90_put_att(file_id, var_id(4), LONG_NAME, AINF_NAME)
status = nf90_put_att(file_id, var_id(5), LONG_NAME, LS_NAME)
status = nf90_put_att(file_id, var_id(6), LONG_NAME, HCID_NAME)

status = nf90_put_att(file_id, var_id(1), UNITS, UTM_X_UNITS)
status = nf90_put_att(file_id, var_id(2), UNITS, UTM_Y_UNITS)
status = nf90_put_att(file_id, var_id(3), UNITS, DEM_UNITS)
status = nf90_put_att(file_id, var_id(4), UNITS, AINF_UNITS)
status = nf90_put_att(file_id, var_id(5), UNITS, LS_UNITS)
status = nf90_put_att(file_id, var_id(6), UNITS, HCID_UNITS)

status = nf90_put_att(file_id, var_id(1),'_FillValue', UTM_X_FillValue)
status = nf90_put_att(file_id, var_id(2),'_FillValue', UTM_Y_FillValue)
status = nf90_put_att(file_id, var_id(3),'_FillValue', DEM_FillValue)
status = nf90_put_att(file_id, var_id(4),'_FillValue', AINF_FillValue)
status = nf90_put_att(file_id, var_id(5),'_FillValue', LS_FillValue)
status = nf90_put_att(file_id, var_id(6),'_FillValue', HCID_FillValue)

status = nf90_enddef(file_id) 
status = nf90_put_var(file_id, var_id(1), utm_x,  &
     start = (/1, 1/), count = (/N, M/))
status = nf90_put_var(file_id, var_id(2), utm_y,  &
     start = (/1, 1/), count = (/N, M/))
status = nf90_put_var(file_id, var_id(3), topo)
status = nf90_put_var(file_id, var_id(4), ainf)
status = nf90_put_var(file_id, var_id(5), ls)
status = nf90_put_var(file_id, var_id(6), hcid)
status = nf90_close(file_id)
end if

call close_dtm()

end subroutine bb2ncdf_sr
